@@ -1,6 +1,18 @@
 import logging
+import os
 from pathlib import Path
-from .measures import ClassificationMeasures
+from typing import Union
+
+
+def get_seed() -> Union[int, None]:
+    seed = os.environ.get("PYHARD_SEED")
+    if seed is None:
+        return seed
+    else:
+        try:
+            return int(seed)
+        except ValueError:
+            return None
 
 
 log_file = Path(__file__).parents[2] / "graphene.log"
