@@ -1,19 +1,25 @@
 import io
 from abc import ABC, abstractmethod
-from typing import Union, Optional, Tuple
+from collections import OrderedDict
 from numbers import Number
+from typing import Union, Optional, Tuple
 
 import holoviews as hv
 import numpy as np
 import pandas as pd
 import panel as pn
+from bokeh.settings import settings
 from bokeh.themes import Theme
 from holoviews import opts, streams
-from shapely.geometry import Point, Polygon
+from shapely.geometry import Point, Polygon, MultiPolygon, MultiPoint
 
 from .context import Workspace
 
 
+settings.resources = 'cdn'
+settings.resources = 'inline'
+
+
 dark_color = '#292929'
 light_color = '#ffffff'
 footprint_colors = {'good': '#58D68D', 'best': '#9B59B6'}
@@ -151,11 +157,11 @@ class BaseMeasures(ABC):
         if measures_list is None:
             measures_list = self._measures_dict.keys()
         elif isinstance(measures_list, list):
-            measures_list = list(set(measures_list) & set(self._measures_dict.keys()))
+            measures_list = sorted(list(set(measures_list) & set(self._measures_dict.keys())))
         else:
             raise TypeError(f"Expected type list for parameter 'measures_list', not '{type(measures_list)}'")
 
-        results = {}
+        results = OrderedDict()
         for k in measures_list:
             self.logger.info(f"Calculating measure {repr(k)}")
             results[k] = self._call_method(self._measures_dict[k])
@@ -188,7 +194,7 @@ class BaseLearner(ABC):
         Estimates the Instance Hardness value.
         """
         L = len(self.predicted_proba.columns)
-        IH = 1 - (1/L)*self.predicted_proba.sum(axis=1).values
+        IH = 1 - (1 / L) * self.predicted_proba.sum(axis=1).values
         return IH
 
     @abstractmethod
@@ -211,8 +217,8 @@ class BaseLearner(ABC):
 
 
 class BaseApp:
-    _tabs_id = ['Instance Space', 'Footprint performance', 'Selection explorer']
-    _tabs_inner_id = ['Statistics', 'Features', 'Meta-features']
+    _tabs_id = ['Instance Space', 'Footprint performance', 'Distributions', 'Data explorer']
+    _tabs_inner_id = ['Meta-features', 'Features']
 
     data: pd.DataFrame
 
@@ -229,14 +235,25 @@ class BaseApp:
         self._page_tabs = pn.Tabs(*[(name, None) for name in self._tabs_id], dynamic=False)
         self._inner_tabs = pn.Tabs(*[(name, None) for name in self._tabs_inner_id], dynamic=True, tabs_location='left')
         self._page_tabs[2] = (self._tabs_id[2], self._inner_tabs)
+        self.all_cols = (
+                workspace.data.columns.to_list() +
+                workspace.extended_metadata.columns.to_list() +
+                workspace.is_coordinates.columns.to_list()
+        )
+        self.output_col = workspace.data.columns[-1]
+
+    @staticmethod
+    def footprint2polygons(footprint: np.ndarray) -> MultiPolygon:
+        poly_list = np.split(footprint, np.argwhere(np.isnan(footprint).any(axis=1)).flatten())
+        return MultiPolygon(list(map(lambda x: Polygon(x[~np.isnan(x).any(axis=1)]), poly_list)))
 
     def footprint_area(self, algo):
         try:
-            border_points_good = self.workspace.footprints.xs([algo, 'good']).values
+            border_points_good = self.workspace.footprints.xs((algo, 'good')).values
         except KeyError:
             border_points_good = np.array([[0, 0]])
         try:
-            border_points_best = self.workspace.footprints.xs([algo, 'best']).values
+            border_points_best = self.workspace.footprints.xs((algo, 'best')).values
         except KeyError:
             border_points_best = np.array([[0, 0]])
 
@@ -257,7 +274,7 @@ class BaseApp:
                                                                   show_legend=True)
         return footprint_good * footprint_best
 
-    def select_instances(self):
+    def select_instances(self) -> Union[pd.Series, pd.DataFrame]:
         if self.bbox is None:
             return pd.Series(False, index=self.data.index)
         x, y = list(self.bbox.keys())
@@ -273,6 +290,16 @@ class BaseApp:
         mask = self.data[[x, y]].apply(lambda p: polygon.contains(Point(p[0], p[1])), raw=True, axis=1)
         return mask
 
+    def select_footprint_instances(self, ftype: str = 'good') -> list:
+        Z = self.workspace.is_coordinates.values
+        algo = self.w_footprint_algo.value
+        try:
+            boundary = self.workspace.footprints.xs([algo, ftype]).values
+        except KeyError:
+            return [False] * Z.shape[0]
+        poly = self.footprint2polygons(boundary)
+        return [poly.contains(p) or poly.boundary.contains(p) for p in MultiPoint(Z)]
+
     @abstractmethod
     def data_space(self, color: str, range_limits: Tuple[Number, Number], autorange_on: bool) -> hv.Scatter:
         pass
@@ -293,17 +320,20 @@ class BaseApp:
             return self.instance_space(color, lim, autorange_on)
 
         def selection_callback1(bbox, region_element, selection_expr, resetting):
-            self.bbox = bbox
             if resetting:
-                self.bbox = None
-            self.populate_tabs()
+                bbox = None
+
+            if self.bbox is not bbox:
+                self.bbox = bbox
+                self.populate_tabs()
+
             return hv.Polygons([[[0, 0]]])
 
         @pn.depends(footprint=self.w_footprint_algo.param.value, fp_on=self.w_footprint_on.param.value)
         def selection_callback2(bbox, region_element, selection_expr, footprint, fp_on):
-            self.bbox = bbox
-
-            self.populate_tabs()
+            if self.bbox is not bbox:
+                self.bbox = bbox
+                self.populate_tabs()
 
             if fp_on:
                 return self.footprint_area(footprint)
@@ -331,10 +361,33 @@ class BaseApp:
             sio.seek(0)
             return sio
 
-        button = pn.widgets.FileDownload(embed=False, auto=True, callback=file_cb,
-                                         filename='selection.csv',
-                                         label='Save selected points',
-                                         button_type='primary')
+        button_download = pn.widgets.FileDownload(embed=False,
+                                                  auto=True,
+                                                  callback=file_cb,
+                                                  filename='selection.csv',
+                                                  label='Save selected points',
+                                                  button_type='primary')
+
+        def file_fp_cb(ftype: str = 'good'):
+            mask = self.select_footprint_instances(ftype)
+            df = self.workspace.data[mask]
+            sio = io.StringIO()
+            df.to_csv(sio)
+            sio.seek(0)
+            return sio
+
+        button_download_fp_good = pn.widgets.FileDownload(embed=False,
+                                                          auto=True,
+                                                          callback=lambda: file_fp_cb('good'),
+                                                          filename='good_footprint_instances.csv',
+                                                          label='Save good footprint points',
+                                                          button_type='default')
+        button_download_fp_best = pn.widgets.FileDownload(embed=False,
+                                                          auto=True,
+                                                          callback=lambda: file_fp_cb('best'),
+                                                          filename='best_footprint_instances.csv',
+                                                          label='Save best footprint points',
+                                                          button_type='default')
 
         layout = (dmap1 * sel1_dmap + dmap2 * sel2_dmap).cols(2).opts(
             opts.Layout(shared_axes=False, shared_datasource=True, framewise=True),
@@ -345,7 +398,7 @@ class BaseApp:
                                 pn.Row(pn.Spacer(), height=20),
                                 '## Footprint', self.w_footprint_on, self.w_footprint_algo,
                                 pn.Row(pn.Spacer(), height=20),
-                                '## Selection', button,
+                                '## Selection', button_download, button_download_fp_good, button_download_fp_best,
                                 background=light_color)
         gspec[0, 1:5] = layout
         self._page_tabs[0] = (self._tabs_id[0], gspec)
@@ -361,7 +414,13 @@ class BaseApp:
     def populate_tabs(self):
         pass
 
-    def show(self, port=5001, show=True):
-        tmpl = self.get_pane()
-        pn.serve(tmpl, port=port, show=show, title='Instance Hardness',
-                 websocket_origin=[f'127.0.0.1:{port}', f'localhost:{port}'])
+    def show(self, port: int = 5001, show: bool = True, ntries: int = 5):
+        self.populate_tabs()
+        for _ in range(ntries):
+            try:
+                pn.serve(self.get_pane, port=port, show=show, title='Instance Hardness',
+                         websocket_origin=[f'127.0.0.1:{port}', f'localhost:{port}'])
+                break
+            except OSError:
+                print(f"Port {port} already in use. Retrying with next port...")
+                port += 1
