@@ -2,13 +2,17 @@
 Module that provides methods for assessing performance of classifiers in the portfolio.
 """
 
+import inspect
 import logging
+import os
+import sys
 import time
 import warnings
 
 import hyperopt
 import numpy as np
 import pandas as pd
+from sklearn.calibration import CalibratedClassifierCV
 from sklearn.dummy import DummyClassifier
 from sklearn.ensemble import RandomForestClassifier, GradientBoostingClassifier, BaggingClassifier
 from sklearn.exceptions import ConvergenceWarning
@@ -18,16 +22,16 @@ from sklearn.naive_bayes import GaussianNB
 from sklearn.neural_network import MLPClassifier
 from sklearn.preprocessing import OneHotEncoder
 from sklearn.preprocessing import StandardScaler
-from sklearn.svm import SVC
+from sklearn.svm import SVC, LinearSVC
 
-from . import metrics
+from . import metrics, get_seed
 from .base import BaseLearner
 from .hpo import find_best_params
 from .metrics import loss_threshold
 
 
 _clf_dict = {
-    'svc_linear': SVC,
+    'svc_linear': LinearSVC,
     'svc_rbf': SVC,
     'random_forest': RandomForestClassifier,
     'gradient_boosting': GradientBoostingClassifier,
@@ -39,8 +43,7 @@ _clf_dict = {
 }
 
 _overlap_params = {
-    'svc_linear': {'probability': True, 'kernel': 'linear'},
-    'svc_rbf': {'probability': True, 'kernel': 'rbf'},
+    'svc_rbf': {'probability': False, 'kernel': 'rbf'},
     'mlp': {'solver': 'lbfgs'}
 }
 
@@ -84,9 +87,6 @@ class Classifier(BaseLearner):
         if new_params is None:
             new_params = _overlap_params.copy()
         else:
-            new_params['svc_linear'] = {**new_params.get('svc_linear'), **_overlap_params['svc_linear']} if \
-                new_params.get('svc_linear') is not None else _overlap_params['svc_linear']
-
             new_params['svc_rbf'] = {**new_params.get('svc_rbf'), **_overlap_params['svc_rbf']} if \
                 new_params.get('svc_rbf') is not None else _overlap_params['svc_rbf']
 
@@ -122,10 +122,16 @@ class Classifier(BaseLearner):
         else:
             raise ValueError("'clf' parameter must be either callable or a valid classifier name string")
 
+        seed = get_seed()
+        if hyper_param_optm:
+            os.environ["HYPEROPT_FMIN_SEED"] = repr(seed) if seed is not None else ""
+        if 'random_state' in inspect.signature(algo).parameters and seed is not None:
+            kwargs = {**kwargs, **{'random_state': seed}}
+
         if verbose:
             self.logger.setLevel(logging.DEBUG)
 
-        kf = StratifiedKFold(n_splits=n_folds, shuffle=True, random_state=0)
+        kf = StratifiedKFold(n_splits=n_folds, shuffle=True, random_state=seed)
 
         score = np.zeros((self.N, n_iter))
         proba = np.zeros((self.N, n_iter))
@@ -156,13 +162,18 @@ class Classifier(BaseLearner):
                 else:
                     clf = algo(**kwargs)
 
-                clf = clf.fit(X_train, y_train)
+                # clf = clf.fit(X_train, y_train)
+                self.logger.info("Calibrating probabilities")
+                calibrated_clf = CalibratedClassifierCV(base_estimator=clf, method='sigmoid',
+                                                        cv=3, ensemble=False, n_jobs=-1)
+                calibrated_clf.fit(X_train, y_train)
 
-                y_pred = clf.predict_proba(X_test)
+                y_pred = calibrated_clf.predict_proba(X_test)
                 score[test_index, i], proba[test_index, i] = self.score(metric=metric, y_true=self.y[test_index],
-                                                                        y_pred=y_pred, classes_order=clf.classes_)
+                                                                        y_pred=y_pred,
+                                                                        classes_order=calibrated_clf.classes_)
 
-                fold_accuracy = clf.score(X_test, self.y[test_index])
+                fold_accuracy = calibrated_clf.score(X_test, self.y[test_index])
                 accuracy += fold_accuracy
                 self.logger.info(f"Test fold mean accuracy: {fold_accuracy}")
 
@@ -181,8 +192,10 @@ class Classifier(BaseLearner):
 
     def run_all(self, metric='logloss', n_folds=10, n_iter=10, algo_list=None, parameters=None,
                 hyper_param_optm=False, hpo_evals=100, hpo_timeout=90, verbose=False):
-        warnings.filterwarnings(action='ignore', module='sklearn', category=ConvergenceWarning)
-        warnings.filterwarnings(action='ignore', module='sklearn', category=UserWarning)
+        if not sys.warnoptions:
+            warnings.filterwarnings(action='ignore', module='sklearn', category=ConvergenceWarning)
+            warnings.filterwarnings(action='ignore', module='sklearn', category=UserWarning)
+            os.environ["PYTHONWARNINGS"] = 'ignore::UserWarning,ignore:::sklearn'
 
         if hyper_param_optm:
             self.logger.info("Hyper parameter optimization enabled")
