@@ -4,11 +4,12 @@ Python package.
 """
 
 import sys
+
 import numpy as np
 from hyperopt import hp, tpe, fmin, space_eval, STATUS_OK
 from hyperopt.pyll import scope
-from sklearn.model_selection import cross_val_score
 from sklearn.base import is_classifier, is_regressor
+from sklearn.model_selection import cross_val_score
 
 _progressbar = True
 
@@ -26,9 +27,9 @@ def classifier_hp_space(name, **kwargs):
 def _objective(predictor, params, X, y):
     predictor = predictor(**params)
     if is_classifier(predictor):
-        score = cross_val_score(predictor, X, y, cv=3, scoring='f1_micro').mean()
+        score = cross_val_score(predictor, X, y, cv=3, scoring='f1_micro', n_jobs=-1).mean()
     elif is_regressor(predictor):
-        score = cross_val_score(predictor, X, y, cv=3, scoring='neg_median_absolute_error').mean()
+        score = cross_val_score(predictor, X, y, cv=3, scoring='neg_median_absolute_error', n_jobs=-1).mean()
     else:
         raise TypeError("Predictor must be either a classifier or a regressor.")
     return {'loss': -score, 'status': STATUS_OK}
@@ -65,10 +66,9 @@ def _svc_linear_hp_space():
     https://scikit-learn.org/stable/modules/generated/sklearn.svm.SVC.html
     :return: SVM Linear parameter search space
     """
-    space = {'kernel': 'linear',
-             'probability': True,
-             'C': hp.loguniform('C', np.log(1e-3), np.log(1e3))
-             }
+    space = {
+        'C': hp.loguniform('C', np.log(1e-3), np.log(1e3))
+    }
     return space
 
 
