@@ -8,6 +8,7 @@ import warnings
 from importlib import import_module
 from inspect import signature, Parameter
 from pathlib import Path
+from typing import Union, Tuple
 
 import pandas as pd
 from deprecation import deprecated
@@ -32,7 +33,8 @@ _mcr_envvar_value = "/usr/local/MATLAB/MATLAB_Runtime/v98/runtime/glnxa64:" \
                     "/usr/local/MATLAB/MATLAB_Runtime/v98/extern/bin/glnxa64"
 
 
-def build_metadata(data, rootdir=' ', problem='classification', output_col=None, save=True, return_ih=False, **kwargs):
+def build_metadata(data: pd.DataFrame, problem: str = 'classification', output_col: str = None,
+                   return_ih: bool = False, **kwargs) -> Union[Tuple[pd.DataFrame, pd.DataFrame], pd.DataFrame]:
     if problem == 'classification':
         measures = ClassificationMeasures(data, labels_col=output_col)
         learners = Classifier(data, labels_col=output_col)
@@ -43,10 +45,10 @@ def build_metadata(data, rootdir=' ', problem='classification', output_col=None,
     else:
         raise ValueError(f"Unknown type of problem: '{problem}'.")
 
-    m_params = {k: kwargs.get(k) for k in get_param_names(measures.calculate_all)}
+    m_params = {k: kwargs.get(k) for k in get_param_names(measures.calculate_all) if kwargs.get(k) is not None}
     df_measures = measures.calculate_all(**m_params)
 
-    c_params = {k: kwargs.get(k) for k in get_param_names(learners.run_all)}
+    c_params = {k: kwargs.get(k) for k in get_param_names(learners.run_all) if kwargs.get(k) is not None}
     df_algo = learners.run_all(**c_params)
 
     df_metadata = pd.concat([df_measures, df_algo], axis=1)
@@ -54,18 +56,10 @@ def build_metadata(data, rootdir=' ', problem='classification', output_col=None,
     df_metadata.insert(0, instances_index, range(1, n_inst + 1))
     df_metadata.set_index(instances_index, inplace=True)
 
-    path = Path(rootdir)
-    if save:
-        if path.is_dir():
-            df_metadata.to_csv(path / metadata_file)
-        else:
-            warnings.warn("Invalid directory '{0}'. File will not be saved.".format(path))
     if return_ih:
         ih_values = learners.estimate_ih()
         df_ih = pd.DataFrame({'instance_hardness': ih_values},
                              index=pd.Index(range(1, n_inst + 1), name=instances_index))
-        if save:
-            df_ih.to_csv(path / ih_file)
         return df_metadata, df_ih
     else:
         return df_metadata
@@ -127,7 +121,7 @@ def update_options(d1: dict, d2: dict):
     return d1
 
 
-def run_isa(rootdir: Path, settings=None, save_output=True, rotation_adjust=True):
+def run_isa(rootdir: Path, metadata=None, settings=None, save_output=True, rotation_adjust=True):
     if rotation_adjust is None:
         rotation_adjust = False
 
@@ -142,7 +136,9 @@ def run_isa(rootdir: Path, settings=None, save_output=True, rotation_adjust=True
         raise FileNotFoundError(f"File 'options.json' not found in specified path {repr(str(rootdir))}.")
 
     meta_path = rootdir / metadata_file
-    if meta_path.is_file():
+    if isinstance(metadata, pd.DataFrame):
+        pass
+    elif meta_path.is_file():
         metadata = pd.read_csv(meta_path, index_col='instances')
     else:
         raise FileNotFoundError(f"File 'metadata.csv' not found in specified path {repr(str(rootdir))}.")
