@@ -5,6 +5,7 @@ import holoviews as hv
 import numpy as np
 import pandas as pd
 import panel as pn
+from bokeh.core.enums import MarkerType
 from bokeh.models import HoverTool
 from holoviews import opts, dim
 from pandas.api.types import is_integer_dtype, is_float_dtype
@@ -43,7 +44,7 @@ class AppClassification(BaseApp):
 
         data = df_is.join(df_dataset)
         self.data = data.join(df_metadata)
-        self.mlist = ['circle', 'triangle', 'square', 'diamond', 'asterisk', 'hex', '+', 'x']
+
         self.data_dims = df_dataset.columns.to_list()
         self.class_label = self.data_dims[2]
         self.meta_dims = df_metadata.columns.to_list()
@@ -55,22 +56,31 @@ class AppClassification(BaseApp):
         self.data_kdims = self.data_dims[0]
         self.data_vdims = [self.data_dims[1], self.class_label] + self.meta_dims
 
+        # Markers
+        markers_list = ['circle', 'triangle', 'square', 'diamond', 'asterisk', 'hex', '+', 'x']
+        if df_dataset[self.class_label].nunique() > len(markers_list):
+            self.marker = dim(self.class_label).categorize(list(MarkerType))
+        else:
+            self.marker = dim(self.class_label).categorize(markers_list)
+
         # Panel widgets
         self.w_color = pn.widgets.Select(options=self.meta_dims + [self.class_label], value=self.meta_dims[0])
         self.w_color_range = pn.widgets.RangeSlider(start=0, end=20, value=(0, 5), step=0.5)
         self.w_checkbox = pn.widgets.Checkbox(name='manual colorbar range', value=False)
         self.w_footprint_on = pn.widgets.Checkbox(name='draw footprint area', value=True)
 
+        # Data explorer tab widgets
+        self.w_var_y = pn.widgets.Select(name='y-axis', options=self.all_cols, value='z_2')
+        self.w_var_x = pn.widgets.Select(name='x-axis', options=self.all_cols, value='z_1')
+        self.w_var_c = pn.widgets.Select(name='Color', options=self.all_cols, value=self.output_col)
+        self.w_var_cm = pn.widgets.RadioButtonGroup(name='Color map', options=['Continuous', 'Discrete'],
+                                                    button_type='default', value='Continuous')
+        self.w_var_s = pn.widgets.DiscreteSlider(name='Size', options=list(range(1, 16)), value=7)
+
         val = 'instance_hardness' if 'instance_hardness' in df_footprint.index else ''
         self.w_footprint_algo = pn.widgets.Select(options=df_footprint.index.unique(level='algo').to_list(), value=val)
 
-        self.populate_tabs()
-
-    @staticmethod
-    @DeprecationWarning
-    def footprint2polygons(footprint: np.ndarray):
-        poly_list = np.split(footprint, np.argwhere(np.isnan(footprint).any(axis=1)).flatten())
-        return MultiPolygon(list(map(lambda x: Polygon(x[~np.isnan(x).any(axis=1)]), poly_list)))
+        # self.populate_tabs()
 
     @classmethod
     @DeprecationWarning
@@ -112,58 +122,68 @@ class AppClassification(BaseApp):
         if not autorange_on:
             lim = (np.nan, np.nan)
         cmap = self.cmap
-        # hover_list = [c] + hover_list
-        # tooltips = [(s, '@' + s) for s in hover_list]
-        # hover = HoverTool(tooltips=tooltips)
-        scatter1 = hv.Scatter(self.data, kdims=self.data_kdims, vdims=self.data_vdims
-                              ).opts(responsive=True, aspect=1.1, color=c,
-                                     cmap=cmap, show_grid=True,
-                                     marker=dim(self.class_label).categorize(self.mlist),
-                                     tools=['lasso_select', 'box_select', 'hover'],
-                                     size=7, framewise=True, colorbar=True, clim=lim)
+        hover_list = [(self.output_col, self.output_col),
+                      ('Component 1', 'Component1'),
+                      ('Component 2', 'Component2'),
+                      ('instance hardness', 'instance_hardness'),
+                      (' '.join(str.split(c, '_', 1)), c)]
+        hover_list = list(dict.fromkeys(hover_list))
+        tooltips = [("index", "$index")] + [(s[0], '@' + s[1]) for s in hover_list]
+        hover = HoverTool(tooltips=tooltips)
+        scatter1 = hv.Scatter(self.data,
+                              kdims=self.data_kdims,
+                              vdims=self.data_vdims
+                              ).opts(responsive=True,
+                                     aspect=1.1,
+                                     color=c,
+                                     cmap=cmap,
+                                     show_grid=True,
+                                     marker=self.marker,
+                                     tools=['lasso_select', 'box_select', hover],
+                                     size=7,
+                                     framewise=True,
+                                     colorbar=True,
+                                     clim=lim)
         return scatter1
 
     def instance_space(self, c, lim, autorange_on):
         if not autorange_on:
             lim = (np.nan, np.nan)
         cmap = self.cmap
-        # hover_list = [c] + hover_list
-        # tooltips = [(s, '@' + s) for s in hover_list]
-        # hover = HoverTool(tooltips=tooltips)
-        scatter2 = hv.Scatter(self.data, kdims=self.is_kdims, vdims=self.is_vdims
-                              ).opts(responsive=True, aspect=1.1, color=c,
-                                     cmap=cmap, show_grid=True,
-                                     marker=dim(self.class_label).categorize(self.mlist),
-                                     tools=['lasso_select', 'box_select', 'hover'],
-                                     size=7, framewise=True, colorbar=True, clim=lim)
+        hover_list = [(self.output_col, self.output_col),
+                      ('z1', 'z_1'),
+                      ('z2', 'z_2'),
+                      ('instance hardness', 'instance_hardness'),
+                      (' '.join(str.split(c, '_', 1)), c)]
+        hover_list = list(dict.fromkeys(hover_list))
+        tooltips = [("index", "$index")] + [(s[0], '@' + s[1]) for s in hover_list]
+        hover = HoverTool(tooltips=tooltips)
+        scatter2 = hv.Scatter(self.data,
+                              kdims=self.is_kdims,
+                              vdims=self.is_vdims
+                              ).opts(responsive=True,
+                                     aspect=1.1,
+                                     color=c,
+                                     cmap=cmap,
+                                     show_grid=True,
+                                     marker=self.marker,
+                                     tools=['lasso_select', 'box_select', hover],
+                                     size=7,
+                                     framewise=True,
+                                     colorbar=True,
+                                     clim=lim)
         return scatter2
 
     def populate_tabs(self):
         mask = self.select_instances()
+        if not mask.any():
+            mask = ~mask
         df_selection = self.workspace.data[mask].copy()
+
         output_col = df_selection.columns[-1]
         df_selection.loc[:, output_col] = df_selection.iloc[:, -1].apply(lambda x: str(x))
         cols = df_selection.columns[:-1].to_list()
 
-        # selection dataframe tab
-        if not mask.any():
-            df_table = self.workspace.data[~mask].copy()
-            df_table[output_col] = df_table[output_col].apply(lambda x: str(x))
-        else:
-            df_table = df_selection
-        desc_all = pn.widgets.DataFrame(df_table.describe(include='all'), name='Selection',
-                                        disabled=True, sizing_mode='stretch_both')
-
-        desc = df_table.groupby(output_col).describe(include='all').unstack(1)
-        desc.index = desc.index.set_names(['Feature', 'Statistic'], level=[0, 1])
-        desc = desc.reset_index(level=[0]).pivot(columns='Feature').reorder_levels([1, 0]).sort_index()
-        desc.columns = desc.columns.droplevel(0)
-        desc_class = pn.widgets.DataFrame(desc, name='Selection', disabled=True, sizing_mode='stretch_both',
-                                          show_index=True, hierarchical=True)
-        table = pn.Column(desc_all, "__Descriptive statistics by class__", desc_class, sizing_mode='stretch_width')
-
-        self._inner_tabs[0] = (self._tabs_inner_id[0], table)
-
         # metadata selection tab
         df_feat = self.workspace.metadata[mask].filter(regex='^feature_').melt(var_name='feature')
         boxplot_feat = hv.BoxWhisker(df_feat, kdims='feature', vdims='value').opts(responsive=True,
@@ -172,7 +192,8 @@ class AppClassification(BaseApp):
                                                                                    show_grid=True,
                                                                                    box_fill_color=dim('feature').str(),
                                                                                    cmap='glasbey_cool',
-                                                                                   tools=['hover'])
+                                                                                   tools=['hover'],
+                                                                                   ylim=(0, 1))
 
         df_algo = self.workspace.metadata[mask].filter(regex='^algo_').melt(var_name='algorithm', value_name='logloss')
         boxplot_algo = hv.BoxWhisker(df_algo, kdims='algorithm', vdims='logloss').opts(responsive=True,
@@ -184,7 +205,7 @@ class AppClassification(BaseApp):
                                                                                        cmap='glasbey_cool',
                                                                                        tools=['hover'])
 
-        self._inner_tabs[2] = (self._tabs_inner_id[2],
+        self._inner_tabs[0] = (self._tabs_inner_id[0],
                                pn.Column('# Distribution of the meta-features', pn.Row(boxplot_feat),
                                          '# Distribution of the algorithms', pn.Row(boxplot_algo),
                                          sizing_mode='stretch_both'))
@@ -222,7 +243,7 @@ class AppClassification(BaseApp):
             tooltips = [(output_col, f'@{output_col}'), ('Count', '@Count')]
             bar.opts(responsive=True,
                      aspect=2.5,
-                     multi_level=True,
+                     multi_level=False,
                      stacked=False,
                      show_grid=True,
                      fontscale=1.2,
@@ -232,7 +253,10 @@ class AppClassification(BaseApp):
         @pn.depends(var=w_var_hist.param.value)
         def make_hist(var):
             classes = df_selection.iloc[:, -1].unique()
-            hist = [hv.Histogram(np.histogram(df_selection[df_selection[output_col] == c][var]), label=str(c))
+            a_min = df_selection[var].min()
+            a_max = df_selection[var].max()
+            hist = [hv.Histogram(np.histogram(df_selection[df_selection[output_col] == c][var],
+                                              range=(a_min, a_max), bins=40), label=str(c))
                     for c in classes]
             if len(hist) == 0:
                 hist = hv.Histogram([])
@@ -267,6 +291,37 @@ class AppClassification(BaseApp):
 
         self._inner_tabs[1] = (self._tabs_inner_id[1], accordion)
 
+        # Explorer outer tab
+        df_all = self.workspace.data[mask].copy()
+        df_all = df_all.join(self.workspace.extended_metadata, how='left')
+        df_all = df_all.join(self.workspace.is_coordinates, how='left')
+
+        @pn.depends(x=self.w_var_x.param.value, y=self.w_var_y.param.value, c=self.w_var_c.param.value,
+                    cm=self.w_var_cm.param.value, s=self.w_var_s.param.value)
+        def plot_scatter(x, y, c, cm, s, **kwargs):
+            cm = {'Discrete': 'tab20c', 'Continuous': 'coolwarm'}[cm]
+            return hv.Scatter(df_all[list({x, y, c})], kdims=[x], vdims=[y, c]).opts(responsive=True,
+                                                                                     aspect=1.8,
+                                                                                     color=c,
+                                                                                     cmap=cm,
+                                                                                     show_grid=True,
+                                                                                     size=s,
+                                                                                     framewise=True,
+                                                                                     colorbar=True,
+                                                                                     tools=['hover']
+                                                                                     )
+
+        dmap_scatter = hv.DynamicMap(plot_scatter)
+        dmap_scatter.opts(framewise=True)
+
+        self._page_tabs[3] = (self._tabs_id[3], pn.Row(pn.Column(self.w_var_y,
+                                                                 self.w_var_x,
+                                                                 self.w_var_c,
+                                                                 self.w_var_cm,
+                                                                 self.w_var_s),
+                                                       dmap_scatter,
+                                                       sizing_mode='stretch_height'))
+
 
 class Demo:
     def __init__(self, datadir=None):
